"""Defines the 'views' used by the Django apps for serving pages of the netbox_ptov plugin"""

import django
from ptovnetlab import ptovnetlab as ptvnl
from netbox.views import generic
from netbox_ptov import filtersets, forms, models, tables
from netbox_ptov.models import gns3srv
from django.shortcuts import render, redirect
from django.contrib import messages
import json
import logging


class MessagesHandler(logging.Handler):
    def __init__(self, request, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.request = request

    def emit(self, record):
        try:
            msg = self.format(record)
            messages.info(self.request, msg)
        except Exception:
            self.handleError(record)


def golab(request: forms.golabForm) -> django.http.HttpResponse:
    """Pass the input fields from the golabForm instance to a background job that executes the ptovnetlab.p_to_v function"""
    if request.method == 'POST':
        form = forms.golabForm(request.POST)
        if form.is_valid():
            # Extract form data
            username = form.cleaned_data['username_in']
            password = form.cleaned_data['password_in']
            switchlist = [str(swname) for swname in form.cleaned_data['switchlist_multiplechoice_in']]
            servername = form.cleaned_data['serverselect_in'].name
            projectname = form.cleaned_data['prjname_in']

            # Log initial info
            messages.add_message(request, messages.INFO, f'Switch-list: {switchlist}')
            messages.add_message(request, messages.INFO, f'GNS3 server: {servername}')

            # Create and start background job
            from .jobs import PToVJob
            job = PToVJob()
            job.start(
                request=request,
                username=username,
                password=password,
                switchlist=switchlist,
                servername=servername,
                prjname=projectname
            )
            job.save()

            # Queue the job for background execution
            job.enqueue()

            messages.add_message(
                request, 
                messages.SUCCESS, 
                f'Virtual lab creation started in background. View progress in the <a href="{job.get_absolute_url()}">Jobs</a> section.',
                extra_tags='safe'
            )
            return render(request, 'golab.html', {'form': form})
    else:
        form = forms.golabForm()
        return render(request, 'golab.html', {'form': form})

class gns3srvView(generic.ObjectView):
    """A class to represent the generic view of a gns3srv object."""

    queryset = models.gns3srv.objects.all()


class gns3srvListView(generic.ObjectListView):
    """A class to represent the view of all gns3srv objects."""

    queryset = models.gns3srv.objects.all()
    table = tables.gns3srvTable


class gns3srvEditView(generic.ObjectEditView):
    """A class to represent the edit view of a gns3srv object.
    =============================================================

    Attributes
    ----------
    queryset
    form
    """
    queryset = models.gns3srv.objects.all()
    form = forms.gns3srvForm


class gns3srvDeleteView(generic.ObjectDeleteView):
    """A class to represent the delete view of a gns3srv object.
    =============================================================
    ...

    Attributes
    ----------
    queryset
    """
    queryset = models.gns3srv.objects.all()


class ptovjobView(generic.ObjectView):
    """
    A class to represent the generic view of all ptovjob objects.

    =============================================================

    Attributes
    ----------
    queryset
    """
    queryset = models.ptovjob.objects.all()


class ptovjobListView(generic.ObjectListView):
    """
    A class to represent the list view of all ptovjob objects.

    =============================================================

    Attributes
    ----------
    queryset
    table
    """
    queryset = models.ptovjob.objects.all()
    table = tables.ptovjobTable


class ptovjobEditView(generic.ObjectEditView):
    """
    A class to represent the edit view of a ptovjob object.

    =============================================================

    Attributes
    ----------
    queryset
    form
    """
    queryset = models.ptovjob.objects.all()
    form = forms.ptovjobForm


class ptovjobDeleteView(generic.ObjectDeleteView):
    """
    A class to represent the delete  view of a ptovjob object.

    =============================================================

    Attributes
    ----------
    queryset
    """

    queryset = models.ptovjob.objects.all()


class switchtojobView(generic.ObjectView):
    """
    A class to represent the generic view of all switchtojob objects.

    =============================================================

    Attributes
    ----------
    queryset
    """
    queryset = models.switchtojob.objects.all()


class switchtojobListView(generic.ObjectListView):
    """
    A class to represent the list view of all switchtojob objects.

    =============================================================

    Attributes
    ----------
    queryset
    table
    """

    queryset = models.switchtojob.objects.all()
    table = tables.switchtojobTable


class switchtojobEditView(generic.ObjectEditView):
    """
    A class to represent the edit view of switchtojob objects.

    =============================================================

    Attributes
    =============================================================
    queryset
    form
    """

    queryset = models.switchtojob.objects.all()
    form = forms.switchtojobForm


class switchtojobDeleteView(generic.ObjectDeleteView):
    """
    A class to represent the delete view of a switchtojob object.

    =============================================================

    Attributes
    ----------
    queryset
    """

    queryset = models.switchtojob.objects.all()



from netbox.jobs import JobRunner
from django.contrib import messages
from ptovnetlab import ptovnetlab as ptvnl
import logging

class PToVJob(JobRunner):
    """
    Job for executing p_to_v function in the background.
    """
    class Meta:
        name = "Create Virtual Lab"
        # description = "Creates a virtual lab in GNS3 from NetBox devices"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.request = None

    def start(self, request, username, password, switchlist, servername, prjname):
        """
        Initialize job with request context and parameters.
        """
        self.request = request
        self.username = username
        self.password = password
        self.switchlist = switchlist
        self.servername = servername
        self.prjname = prjname

    def run(self):
        """
        Execute the p_to_v function with job logging.
        """
        # Create a custom logging handler that forwards to both job logs and Django messages
        class MessagesHandler(logging.Handler):
            def __init__(self, job, request):
                super().__init__()
                self.job = job
                self.request = request

            def emit(self, record):
                msg = self.format(record)
                # Log to job
                self.job.log_info(msg)
                # Forward to Django messages if request exists
                if self.request:
                    messages.info(self.request, msg)

        # Set up logging
        logger = logging.getLogger('ptovnetlab')
        handler = MessagesHandler(self, self.request)
        handler.setFormatter(logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s'))
        logger.addHandler(handler)

        try:
            # Execute p_to_v with provided parameters
            result = ptvnl.p_to_v(
                username=self.username,
                passwd=self.password,
                servername=self.servername,
                switchlist=self.switchlist,
                prjname=self.prjname
            )

            # Log success
            self.log_success(f"Project Created: {self.prjname} on {self.servername}")
            if result:
                self.log_info(f"Project URL: {result}")

            return result

        except Exception as e:
            # Log error and re-raise
            self.log_failure(str(e))
            raise

        finally:
            # Clean up logging
            logger.removeHandler(handler)
